import inspect
import functools
import importlib
import os
import numpy as np
import pandas as pd
import json

# from sachima.publish import Publisher


# def api(platform="superset", isRun=False, **kw_api):
#     def wrapper(func):
#         @functools.wraps(func)
#         def api_called(*args, **kw):

#             if platform == "superset":
#                 fpath = inspect.getfile(func)
#                 fname = os.path.split(fpath)[1]
#                 fmod = os.path.splitext(fname)[0]
#                 Publisher.to_superset(
#                     name=kw_api.get("name", fmod),
#                     type_=kw_api.get("type_", "RPC"),
#                     param=fmod,
#                 )
#             else:
#                 pass

#             if isRun:
#                 _result = func(*args, **kw)
#                 return _result

#         return api_called

#     return wrapper



import random
import colorsys
import matplotlib as mpl
import seaborn as sns

mpl.use("Agg")


def get_random_color(pastel_factor=0.5):
    return [
        (x + pastel_factor) / (1.0 + pastel_factor)
        for x in [random.uniform(0, 1.0) for i in [1, 2, 3]]
    ]


def color_distance(c1, c2):
    return sum([abs(x[0] - x[1]) for x in zip(c1, c2)])


def generate_new_color(existing_colors, pastel_factor=0.5):
    max_distance = None
    best_color = None
    for i in range(0, 100):
        color = get_random_color(pastel_factor=pastel_factor)
        if not existing_colors:
            return color
        best_distance = min(
            [color_distance(color, c) for c in existing_colors]
        )
        if not max_distance or best_distance > max_distance:
            max_distance = best_distance
            best_color = color
    return best_color


def rgb2hex(r, g, b):
    return "#{:02x}{:02x}{:02x}".format(r, g, b)


def gen_hex_colors(colorcode, N):
    # (210, 90, 60)"#"
    # colors = sns.light_palette((254, 67, 101), input="husl",n_colors=N)
    # https://seaborn.pydata.org/generated/seaborn.light_palette.html
    colors = sns.light_palette(colorcode, n_colors=N)
    hex_colors = []

    # for i in range(0,N):
    #     colors.append(generate_new_color(colors,pastel_factor=0.0000001) )

    for c in colors:
        hex_colors.append(
            rgb2hex(int(c[0] * 255), int(c[1] * 255), int(c[2] * 255))
        )

    return hex_colors


# Example:
# if __name__ == "__main__":
#     pass
# print(gen_hex_colors(100))
# print(sns.color_palette("Blues"))
# print(colnum_string(98))



import pandas as pd
from sachima import conf
import os
import importlib
from sachima.params import set_sql_params
from sachima.log import logger


class Data:
    def __init__(self, dataname, datatype, params):
        """
        dataname: sql filename
        datatype: db engine  or filetype in str
        """
        self.source = dataname
        self.type = datatype
        logger.info(dataname + str(datatype))
        if datatype in ("xls", "xlsx"):
            self.data = pd.read_excel(
                os.path.join(conf.get("PROJ_DIR"), "data", dataname)
            )
        elif datatype in ("csv", "txt"):
            self.data = pd.read_csv(
                os.path.join(conf.get("PROJ_DIR"), "data", dataname)
            )
        elif datatype in ("api",):
            api_cls = importlib.import_module(
                "services." + dataname, package=".."
            )
            api = api_cls.Api()
            self.data = api.data
        else:
            str_sql = open(
                os.path.join(conf.get("PROJ_DIR"), "sqls", dataname),
                encoding="utf-8",
            ).read()
            sql = set_sql_params(str_sql, params)
            self.data = pd.read_sql(sql, datatype)



import os
from premailer import transform


def apply_html_style(strhtmltablelist,
                     title='Data Reports',
                     cssfile='tohtmlstyle_example.css'):
    css_file_path = os.path.join('views', cssfile)
    if os.path.exists(css_file_path):
        cs = css_file_path
    else:
        cs = os.path.join(os.path.dirname(__file__), 'views', cssfile)

    with open(cs, 'r', encoding='utf-8') as f:
        css = f.read()
    title = '<div class="tabletitle">'+title+'</div>'
    body = ''.join(strhtmltablelist)
    body = body.replace('class="dataframe"', 'class="GenericTable"')
    res = ''.join([
        '<html><head><style>',
        css,
        '</style></head><body>',
        title,
        body,
        '</body>'
    ])
    return transform(res)
